@@ -137,11 +137,14 @@ enum lttng_error_code {
 	LTTNG_ERR_PID_TRACKED            = 114, /* PID already tracked */
 	LTTNG_ERR_PID_NOT_TRACKED        = 115, /* PID not tracked */
 	LTTNG_ERR_INVALID_CHANNEL_DOMAIN = 116, /* Invalid channel domain */
-	LTTNG_ERR_OVERFLOW		 = 117, /* Overflow occured. */
+	LTTNG_ERR_OVERFLOW		 = 117, /* Overflow occurred. */
 	LTTNG_ERR_SESSION_NOT_STARTED    = 118, /* Session not started */
 	LTTNG_ERR_LIVE_SESSION           = 119, /* Live session unsupported */
 	LTTNG_ERR_PER_PID_SESSION        = 120, /* Per-PID sessions unsupported */
 	LTTNG_ERR_KERN_CONTEXT_UNAVAILABLE = 121, /* Context unavailable on this kernel */
+	LTTNG_ERR_REGEN_STATEDUMP_FAIL   = 122, /* Failed to regenerate the state dump */
+	LTTNG_ERR_REGEN_STATEDUMP_NOMEM  = 123, /* Failed to regenerate the state dump, not enough memory */
+	LTTNG_ERR_NOT_SNAPSHOT_SESSION   = 124, /* Session is not in snapshot mode. */
 
 	/* MUST be last element */
 	LTTNG_ERR_NR,                           /* Last element */
