@@ -150,52 +150,6 @@ error_create:
 	return NULL;
 }
 
-static int mi_list_output(void)
-{
-	int ret;
-	struct lttng_snapshot_output *s_iter;
-	struct lttng_snapshot_output_list *list;
-
-	assert(writer);
-
-	ret = lttng_snapshot_list_output(current_session_name, &list);
-	if (ret < 0) {
-		goto error;
-	}
-
-	ret = mi_lttng_snapshot_output_session_name(writer, current_session_name);
-	if (ret) {
-		ret = CMD_ERROR;
-		goto end;
-	}
-
-	while ((s_iter = lttng_snapshot_output_list_get_next(list)) != NULL) {
-		ret = mi_lttng_snapshot_list_output(writer, s_iter);
-		if (ret) {
-			ret = CMD_ERROR;
-			goto end;
-		}
-	}
-
-
-	/* Close snapshot snapshots element */
-	ret = mi_lttng_writer_close_element(writer);
-	if (ret) {
-		ret = CMD_ERROR;
-		goto end;
-	}
-
-	/* Close snapshot session element */
-	ret = mi_lttng_writer_close_element(writer);
-	if (ret) {
-		ret = CMD_ERROR;
-	}
-end:
-	lttng_snapshot_output_list_destroy(list);
-error:
-	return ret;
-}
-
 static int list_output(void)
 {
 	int ret, output_seen = 0;
@@ -209,6 +163,15 @@ static int list_output(void)
 
 	MSG("Snapshot output list for session %s", current_session_name);
 
+	if (lttng_opt_mi) {
+		ret = mi_lttng_snapshot_output_session_name(writer,
+				current_session_name);
+		if (ret) {
+			ret = CMD_ERROR;
+			goto end;
+		}
+	}
+
 	while ((s_iter = lttng_snapshot_output_list_get_next(list)) != NULL) {
 		MSG("%s[%" PRIu32 "] %s: %s (max-size: %" PRId64 ")", indent4,
 				lttng_snapshot_output_get_id(s_iter),
@@ -216,8 +179,30 @@ static int list_output(void)
 				lttng_snapshot_output_get_ctrl_url(s_iter),
 				lttng_snapshot_output_get_maxsize(s_iter));
 		output_seen = 1;
+		if (lttng_opt_mi) {
+			ret = mi_lttng_snapshot_list_output(writer, s_iter);
+			if (ret) {
+				ret = CMD_ERROR;
+				goto end;
+			}
+		}
 	}
 
+	if (lttng_opt_mi) {
+		/* Close snapshot snapshots element */
+		ret = mi_lttng_writer_close_element(writer);
+		if (ret) {
+			ret = CMD_ERROR;
+			goto end;
+		}
+
+		/* Close snapshot session element */
+		ret = mi_lttng_writer_close_element(writer);
+		if (ret) {
+			ret = CMD_ERROR;
+		}
+	}
+end:
 	lttng_snapshot_output_list_destroy(list);
 
 	if (!output_seen) {
@@ -228,50 +213,6 @@ error:
 	return ret;
 }
 
-/*
- * Delete output by ID (machine interface version).
- */
-static int mi_del_output(uint32_t id, const char *name)
-{
-	int ret;
-	struct lttng_snapshot_output *output = NULL;
-
-	assert(writer);
-
-	output = lttng_snapshot_output_create();
-	if (!output) {
-		ret = CMD_FATAL;
-		goto error;
-	}
-
-	if (name) {
-		ret = lttng_snapshot_output_set_name(name, output);
-	} else if (id != UINT32_MAX) {
-		ret = lttng_snapshot_output_set_id(id, output);
-	} else {
-		ret = CMD_ERROR;
-		goto error;
-	}
-	if (ret < 0) {
-		ret = CMD_FATAL;
-		goto error;
-	}
-
-	ret = lttng_snapshot_del_output(current_session_name, output);
-	if (ret < 0) {
-		goto error;
-	}
-
-	ret = mi_lttng_snapshot_del_output(writer, id, name, current_session_name);
-	if (ret) {
-		ret = CMD_ERROR;
-	}
-
-error:
-	lttng_snapshot_output_destroy(output);
-	return ret;
-}
-
 /*
  * Delete output by ID.
  */
@@ -312,53 +253,12 @@ static int del_output(uint32_t id, const char *name)
 				name, current_session_name);
 	}
 
-error:
-	lttng_snapshot_output_destroy(output);
-	return ret;
-}
-
-/*
- * Add output from the user URL (machine interface).
- */
-static int mi_add_output(const char *url)
-{
-	int ret;
-	struct lttng_snapshot_output *output = NULL;
-	char name[NAME_MAX];
-	const char *n_ptr;
-
-	if (!url && (!opt_data_url || !opt_ctrl_url)) {
-		ret = CMD_ERROR;
-		goto error;
-	}
-
-	output = create_output_from_args(url);
-	if (!output) {
-		ret = CMD_FATAL;
-		goto error;
-	}
-
-	/* This call, if successful, populates the id of the output object. */
-	ret = lttng_snapshot_add_output(current_session_name, output);
-	if (ret < 0) {
-		goto error;
-	}
-
-	n_ptr = lttng_snapshot_output_get_name(output);
-	if (*n_ptr == '\0') {
-		int pret;
-		pret = snprintf(name, sizeof(name), DEFAULT_SNAPSHOT_NAME "-%" PRIu32,
-				lttng_snapshot_output_get_id(output));
-		if (pret < 0) {
-			PERROR("snprintf add output name");
+	if (lttng_opt_mi) {
+		ret = mi_lttng_snapshot_del_output(writer, id, name,
+				current_session_name);
+		if (ret) {
+			ret = CMD_ERROR;
 		}
-		n_ptr = name;
-	}
-
-	ret = mi_lttng_snapshot_add_output(writer, current_session_name, n_ptr,
-			output);
-	if (ret) {
-		ret = CMD_ERROR;
 	}
 
 error:
@@ -410,6 +310,13 @@ static int add_output(const char *url)
 			lttng_snapshot_output_get_id(output), n_ptr,
 			lttng_snapshot_output_get_ctrl_url(output),
 			lttng_snapshot_output_get_maxsize(output));
+	if (lttng_opt_mi) {
+		ret = mi_lttng_snapshot_add_output(writer, current_session_name,
+				n_ptr, output);
+		if (ret) {
+			ret = CMD_ERROR;
+		}
+	}
 error:
 	lttng_snapshot_output_destroy(output);
 	return ret;
@@ -424,11 +331,7 @@ static int cmd_add_output(int argc, const char **argv)
 		goto end;
 	}
 
-	if (lttng_opt_mi) {
-		ret = mi_add_output(argv[1]);
-	} else {
-		ret = add_output(argv[1]);
-	}
+	ret = add_output(argv[1]);
 
 end:
 	return ret;
@@ -447,18 +350,10 @@ static int cmd_del_output(int argc, const char **argv)
 
 	errno = 0;
 	id = strtol(argv[1], &name, 10);
-	if (id == 0 && errno == 0) {
-		if (lttng_opt_mi) {
-			ret = mi_del_output(UINT32_MAX, name);
-		} else {
-			ret = del_output(UINT32_MAX, name);
-		}
+	if (id == 0 && (errno == 0 || errno == EINVAL)) {
+		ret = del_output(UINT32_MAX, name);
 	} else if (errno == 0 && *name == '\0') {
-		if (lttng_opt_mi) {
-			ret = mi_del_output(id, NULL);
-		} else {
-			ret = del_output(id, NULL);
-		}
+		ret = del_output(id, NULL);
 	} else {
 		ERR("Argument %s not recognized", argv[1]);
 		ret = -1;
@@ -473,46 +368,11 @@ static int cmd_list_output(int argc, const char **argv)
 {
 	int ret;
 
-	if (lttng_opt_mi) {
-		ret = mi_list_output();
-	} else {
-		ret = list_output();
-	}
+	ret = list_output();
 
 	return ret;
 }
 
-/*
- * Do a snapshot record with the URL if one is given (machine interface).
- */
-static int mi_record(const char *url)
-{
-	int ret;
-	struct lttng_snapshot_output *output = NULL;
-
-	output = create_output_from_args(url);
-	if (!output) {
-		ret = CMD_FATAL;
-		goto error;
-	}
-
-	ret = lttng_snapshot_record(current_session_name, output, 0);
-	if (ret < 0) {
-		ret = CMD_ERROR;
-		goto error;
-	}
-
-	ret = mi_lttng_snapshot_record(writer, current_session_name, url,
-			opt_ctrl_url, opt_data_url);
-	if (ret) {
-		ret = CMD_ERROR;
-	}
-
-error:
-	lttng_snapshot_output_destroy(output);
-	return ret;
-}
-
 /*
  * Do a snapshot record with the URL if one is given.
  */
@@ -544,6 +404,14 @@ static int record(const char *url)
 				opt_data_url);
 	}
 
+	if (lttng_opt_mi) {
+		ret = mi_lttng_snapshot_record(writer, current_session_name, url,
+				opt_ctrl_url, opt_data_url);
+		if (ret) {
+			ret = CMD_ERROR;
+		}
+	}
+
 error:
 	lttng_snapshot_output_destroy(output);
 	return ret;
@@ -554,18 +422,9 @@ static int cmd_record(int argc, const char **argv)
 	int ret;
 
 	if (argc == 2) {
-		/* With a given URL */
-		if (lttng_opt_mi) {
-			ret = mi_record(argv[1]);
-		} else {
-			ret = record(argv[1]);
-		}
+		ret = record(argv[1]);
 	} else {
-		if (lttng_opt_mi) {
-			ret = mi_record(NULL);
-		} else {
-			ret = record(NULL);
-		}
+		ret = record(NULL);
 	}
 
 	return ret;
@@ -720,9 +579,6 @@ int cmd_snapshot(int argc, const char **argv)
 	command_ret = handle_command(poptGetArgs(pc));
 	if (command_ret) {
 		switch (-command_ret) {
-		case LTTNG_ERR_EPERM:
-			ERR("The session needs to be set in no output mode (--no-output)");
-			break;
 		case LTTNG_ERR_SNAPSHOT_NODATA:
 			WARN("%s", lttng_strerror(command_ret));
 
@@ -777,7 +633,7 @@ end:
 		free(session_name);
 	}
 
-	/* Overwrite ret if an error occured during handle_command */
+	/* Overwrite ret if an error occurred during handle_command */
 	ret = command_ret ? command_ret : ret;
 	poptFreeContext(pc);
 	return ret;
