@@ -16,7 +16,6 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#define _GNU_SOURCE
 #define _LGPL_SOURCE
 #include <ctype.h>
 #include <popt.h>
@@ -65,28 +64,6 @@ static struct poptOption long_options[] = {
 	{ 0, 0, 0, 0, 0, 0, 0, },
 };
 
-/*
- * usage
- */
-static void usage(FILE *ofp, const char *cmd_str)
-{
-	fprintf(ofp, "usage: lttng %s [-k|-u] [OPTIONS]\n", cmd_str);
-	fprintf(ofp, "\n");
-	fprintf(ofp, "If no session is given (-s), the context is added to\n");
-	fprintf(ofp, "the current sesssion. Exactly one domain (-k or -u)\n");
-	fprintf(ofp, "must be specified.\n");
-	fprintf(ofp, "\n");
-	fprintf(ofp, "Options:\n");
-	fprintf(ofp, "  -h, --help               Show this help.\n");
-	fprintf(ofp, "      --list-options       Simple listing of options.\n");
-	fprintf(ofp, "  -s, --session NAME       Apply to session name.\n");
-	fprintf(ofp, "  -k, --kernel             Apply to the kernel tracer.\n");
-	fprintf(ofp, "  -u, --userspace          Apply to the user-space tracer.\n");
-	fprintf(ofp, "  -p, --pid [PID]          Process ID tracker. Leave PID empty when used with --all.\n");
-	fprintf(ofp, "  -a, --all                All PIDs (use with --pid).\n");
-	fprintf(ofp, "\n");
-}
-
 static
 int parse_pid_string(const char *_pid_string,
 		int all, int **_pid_list, int *nr_pids)
@@ -110,7 +87,7 @@ int parse_pid_string(const char *_pid_string,
 		goto error;
 	}
 	if (all) {
-		pid_list = zmalloc(sizeof(*_pid_list));
+		pid_list = zmalloc(sizeof(*pid_list));
 		if (!pid_list) {
 			ERR("Out of memory");
 			retval = CMD_ERROR;
@@ -172,7 +149,7 @@ int parse_pid_string(const char *_pid_string,
 		goto error;
 	}
 
-	/* Copy */
+	/* Reparse string and populate the pid list. */
 	count = 0;
 	one_pid_str = strtok_r(pid_string, ",", &iter);
 	while (one_pid_str != NULL) {
@@ -209,14 +186,14 @@ enum cmd_error_code track_untrack_pid(enum cmd_type cmd_type, const char *cmd_st
 	int nr_pids;
 	struct lttng_domain dom;
 	struct lttng_handle *handle = NULL;
-	int (*lib_func)(struct lttng_handle *handle, int pid);
+	int (*cmd_func)(struct lttng_handle *handle, int pid);
 
 	switch (cmd_type) {
 	case CMD_TRACK:
-		lib_func = lttng_track_pid;
+		cmd_func = lttng_track_pid;
 		break;
 	case CMD_UNTRACK:
-		lib_func = lttng_untrack_pid;
+		cmd_func = lttng_untrack_pid;
 		break;
 	default:
 		ERR("Unknown command");
@@ -237,7 +214,6 @@ enum cmd_error_code track_untrack_pid(enum cmd_type cmd_type, const char *cmd_st
 	ret = parse_pid_string(pid_string, all, &pid_list, &nr_pids);
 	if (ret != CMD_SUCCESS) {
 		ERR("Error parsing PID string");
-		usage(stderr, cmd_str);
 		retval = CMD_ERROR;
 		goto end;
 	}
@@ -259,7 +235,7 @@ enum cmd_error_code track_untrack_pid(enum cmd_type cmd_type, const char *cmd_st
 
 	for (i = 0; i < nr_pids; i++) {
 		DBG("%s PID %d", cmd_str, pid_list[i]);
-		ret = lib_func(handle, pid_list[i]);
+		ret = cmd_func(handle, pid_list[i]);
 		if (ret) {
 			switch (-ret) {
 			case LTTNG_ERR_PID_TRACKED:
@@ -354,7 +330,6 @@ int cmd_track_untrack(enum cmd_type cmd_type, const char *cmd_str,
 	struct mi_writer *writer = NULL;
 
 	if (argc < 1) {
-		usage(stderr, cmd_str);
 		command_ret = CMD_ERROR;
 		goto end;
 	}
@@ -365,7 +340,7 @@ int cmd_track_untrack(enum cmd_type cmd_type, const char *cmd_str,
 	while ((opt = poptGetNextOpt(pc)) != -1) {
 		switch (opt) {
 		case OPT_HELP:
-			usage(stdout, cmd_str);
+			SHOW_HELP();
 			goto end;
 		case OPT_LIST_OPTIONS:
 			list_cmd_options(stdout, long_options);
@@ -375,7 +350,6 @@ int cmd_track_untrack(enum cmd_type cmd_type, const char *cmd_str,
 			opt_pid = 1;
 			break;
 		default:
-			usage(stderr, cmd_str);
 			command_ret = CMD_UNDEFINED;
 			goto end;
 		}
@@ -400,7 +374,6 @@ int cmd_track_untrack(enum cmd_type cmd_type, const char *cmd_str,
 	/* Currently only PID tracker is supported */
 	if (!opt_pid) {
 		ERR("Please specify at least one tracker with its expected arguments");
-		usage(stderr, cmd_str);
 		command_ret = CMD_ERROR;
 		goto end;
 	}
