@@ -15,7 +15,6 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#define _GNU_SOURCE
 #define _LGPL_SOURCE
 #include <assert.h>
 #include <fcntl.h>
@@ -24,7 +23,6 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#include <config.h>
 
 #include <common/error.h>
 #include <common/defaults.h>
@@ -78,8 +76,9 @@ int compat_epoll_create(struct lttng_poll_event *events, int size, int flags)
 	}
 
 	if (!poll_max_size) {
-		ERR("poll_max_size not initialized yet");
-		goto error;
+		if (lttng_poll_set_max_size()) {
+			goto error;
+		}
 	}
 
 	/* Don't bust the limit here */
@@ -210,7 +209,6 @@ int compat_epoll_wait(struct lttng_poll_event *events, int timeout)
 		ERR("Wrong arguments in compat_epoll_wait");
 		goto error;
 	}
-	assert(events->nb_fd >= 0);
 
 	if (events->nb_fd == 0) {
 		errno = EINVAL;
