@@ -1475,14 +1475,17 @@ int lttng_kconsumer_on_recv_stream(struct lttng_consumer_stream *stream)
 		stream->tracefile_size_current = 0;
 
 		if (!stream->metadata_flag) {
-			ret = index_create_file(stream->chan->pathname,
+			struct lttng_index_file *index_file;
+
+			index_file = lttng_index_file_create(stream->chan->pathname,
 					stream->name, stream->uid, stream->gid,
 					stream->chan->tracefile_size,
-					stream->tracefile_count_current);
-			if (ret < 0) {
+					stream->tracefile_count_current,
+					CTF_INDEX_MAJOR, CTF_INDEX_MINOR);
+			if (!index_file) {
 				goto error;
 			}
-			stream->index_fd = ret;
+			stream->index_file = index_file;
 		}
 	}
 
