@@ -1,6 +1,7 @@
 /*
  * Copyright (C) 2014 - Jonathan Rajotte <jonathan.r.julien@gmail.com>
  *                    - Olivier Cotte <olivier.cotte@polymtl.ca>
+ * Copyright (C) 2016 - Jérémie Galarneau <jeremie.galarneau@efficios.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License, version 2 only, as
@@ -23,7 +24,7 @@
 
 #include <common/error.h>
 #include <common/macros.h>
-#include <common/config/config.h>
+#include <common/config/session-config.h>
 #include <lttng/lttng.h>
 
 /* Don't want to reference snapshot-internal.h here */
@@ -39,20 +40,19 @@ struct mi_writer {
  * Version information for the machine interface.
  */
 struct mi_lttng_version {
-	char version[NAME_MAX]; /* Version number of package */
+	char version[LTTNG_NAME_MAX]; /* Version number of package */
 	uint32_t version_major; /* LTTng-Tools major version number */
 	uint32_t version_minor; /* LTTng-Tools minor version number */
 	uint32_t version_patchlevel; /* LTTng-Tools patchlevel version number */
-	char version_commit[NAME_MAX]; /* Commit hash of the current version */
-	char version_name[NAME_MAX];
-	char package_url[NAME_MAX]; /* Define to the home page for this package. */
+	char version_commit[LTTNG_NAME_MAX]; /* Commit hash of the current version */
+	char version_name[LTTNG_NAME_MAX];
+	char package_url[LTTNG_NAME_MAX]; /* Define to the home page for this package. */
 };
 
 /* Strings related to command */
 extern const char * const mi_lttng_element_command;
 extern const char * const mi_lttng_element_command_action;
 extern const char * const mi_lttng_element_command_add_context;
-extern const char * const mi_lttng_element_command_calibrate;
 extern const char * const mi_lttng_element_command_create;
 extern const char * const mi_lttng_element_command_destroy;
 extern const char * const mi_lttng_element_command_disable_channel;
@@ -61,6 +61,10 @@ extern const char * const mi_lttng_element_command_enable_channels;
 extern const char * const mi_lttng_element_command_enable_event;
 extern const char * const mi_lttng_element_command_list;
 extern const char * const mi_lttng_element_command_load;
+extern const char * const mi_lttng_element_command_metadata;
+extern const char * const mi_lttng_element_command_metadata_action;
+extern const char * const mi_lttng_element_command_regenerate;
+extern const char * const mi_lttng_element_command_regenerate_action;
 extern const char * const mi_lttng_element_command_name;
 extern const char * const mi_lttng_element_command_output;
 extern const char * const mi_lttng_element_command_save;
@@ -92,11 +96,6 @@ extern const char * const mi_lttng_element_version_web;
 extern const char * const mi_lttng_element_event_field;
 extern const char * const mi_lttng_element_event_fields;
 
-/* String related to lttng_event_context */
-extern const char * const mi_lttng_context_type_perf_counter;
-extern const char * const mi_lttng_context_type_perf_cpu_counter;
-extern const char * const mi_lttng_context_type_perf_thread_counter;
-
 /* String related to lttng_event_perf_counter_ctx */
 extern const char * const mi_lttng_element_perf_counter_context;
 
@@ -108,6 +107,8 @@ extern const char * const mi_lttng_element_save;
 
 /* Strings related to load command */
 extern const char * const mi_lttng_element_load;
+LTTNG_HIDDEN const char * const mi_lttng_element_load_overrides;
+LTTNG_HIDDEN const char * const mi_lttng_element_load_override_url;
 
 /* General element of mi_lttng */
 extern const char * const mi_lttng_element_empty;
@@ -173,10 +174,6 @@ extern const char * const mi_lttng_loglevel_type_range;
 extern const char * const mi_lttng_loglevel_type_single;
 extern const char * const mi_lttng_loglevel_type_unknown;
 
-/* Sting related to lttng_calibrate */
-extern const char * const mi_lttng_element_calibrate;
-extern const char * const mi_lttng_element_calibrate_function;
-
 /* String related to a lttng_snapshot */
 extern const char * const mi_lttng_element_snapshot_ctrl_url;
 extern const char * const mi_lttng_element_snapshot_data_url;
@@ -185,18 +182,15 @@ extern const char * const mi_lttng_element_snapshot_n_ptr;
 extern const char * const mi_lttng_element_snapshot_session_name;
 extern const char * const mi_lttng_element_snapshots;
 
+/* String related to track/untrack command */
+const char * const mi_lttng_element_track_untrack_all_wildcard;
+
 /* Utility string function  */
 const char *mi_lttng_loglevel_string(int value, enum lttng_domain_type domain);
 const char *mi_lttng_logleveltype_string(enum lttng_loglevel_type value);
 const char *mi_lttng_eventfieldtype_string(enum lttng_event_field_type value);
 const char *mi_lttng_domaintype_string(enum lttng_domain_type value);
 const char *mi_lttng_buffertype_string(enum lttng_buffer_type value);
-const char *mi_lttng_calibratetype_string(enum lttng_calibrate_type val);
-
-/* String related to track/untrack command */
-const char * const mi_lttng_element_track_untrack_targets;
-const char * const mi_lttng_element_track_untrack_pid_target;
-const char * const mi_lttng_element_track_untrack_all_wildcard;
 
 /*
  * Create an instance of a machine interface writer.
@@ -656,19 +650,6 @@ int mi_lttng_targets_open(struct mi_writer *writer);
  */
 int mi_lttng_pid_target(struct mi_writer *writer, pid_t pid, int is_open);
 
-/*
- * Machine interface for struct lttng_calibrate.
- *
- * writer An instance of a machine interface writer.
- *
- * calibrate A lttng_calibrate instance.
- *
- * Returns zero if the element's value could be written.
- * Negative values indicate an error.
- */
-int mi_lttng_calibrate(struct mi_writer *writer,
-		struct lttng_calibrate *calibrate);
-
 /*
  * Machine interface of a context.
  *
