@@ -19,10 +19,9 @@
 #ifndef _LTTNG_USTCONSUMER_H
 #define _LTTNG_USTCONSUMER_H
 
-#include <config.h>
 #include <errno.h>
 
-#include <common/consumer.h>
+#include <common/consumer/consumer.h>
 
 #ifdef HAVE_LIBLTTNG_UST_CTL
 
@@ -58,8 +57,8 @@ void lttng_ustconsumer_close_all_metadata(struct lttng_ht *ht);
 void lttng_ustconsumer_close_metadata(struct lttng_consumer_channel *metadata);
 void lttng_ustconsumer_close_stream_wakeup(struct lttng_consumer_stream *stream);
 int lttng_ustconsumer_recv_metadata(int sock, uint64_t key, uint64_t offset,
-		uint64_t len, struct lttng_consumer_channel *channel,
-		int timer, int wait);
+		uint64_t len, uint64_t version,
+		struct lttng_consumer_channel *channel, int timer, int wait);
 int lttng_ustconsumer_request_metadata(struct lttng_consumer_local_data *ctx,
 		struct lttng_consumer_channel *channel, int timer, int wait);
 int lttng_ustconsumer_sync_metadata(struct lttng_consumer_local_data *ctx,
@@ -68,6 +67,8 @@ void lttng_ustconsumer_flush_buffer(struct lttng_consumer_stream *stream,
 		int producer);
 int lttng_ustconsumer_get_current_timestamp(
 		struct lttng_consumer_stream *stream, uint64_t *ts);
+int lttng_ustconsumer_get_sequence_number(
+		struct lttng_consumer_stream *stream, uint64_t *seq);
 
 #else /* HAVE_LIBLTTNG_UST_CTL */
 
@@ -184,8 +185,8 @@ void lttng_ustconsumer_close_stream_wakeup(struct lttng_consumer_stream *stream)
 }
 static inline
 int lttng_ustconsumer_recv_metadata(int sock, uint64_t key, uint64_t offset,
-		uint64_t len, struct lttng_consumer_channel *channel,
-		int timer)
+		uint64_t len, uint64_t version,
+		struct lttng_consumer_channel *channel, int timer)
 {
 	return -ENOSYS;
 }
@@ -213,6 +214,12 @@ int lttng_ustconsumer_get_current_timestamp(
 	return -ENOSYS;
 }
 static inline
+int lttng_ustconsumer_get_sequence_number(
+		struct lttng_consumer_stream *stream, uint64_t *seq)
+{
+	return -ENOSYS;
+}
+static inline
 int lttng_ustconsumer_get_stream_id(struct lttng_consumer_stream *stream,
 		uint64_t *stream_id)
 {
