@@ -16,7 +16,6 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#define _GNU_SOURCE
 #include <assert.h>
 #include <errno.h>
 #include <stdio.h>
@@ -39,7 +38,7 @@
 #define RANDOM_STRING_LEN	11
 
 /* Number of TAP tests in this file */
-#define NUM_TESTS 11
+#define NUM_TESTS 16
 
 /* For error.h */
 int lttng_opt_quiet = 1;
@@ -49,6 +48,9 @@ int lttng_opt_mi;
 int ust_consumerd32_fd;
 int ust_consumerd64_fd;
 
+/* Global variable required by sessiond objects being linked-in */
+struct lttng_ht *agent_apps_ht_by_sock;
+
 static const char alphanum[] =
 	"0123456789"
 	"ABCDEFGHIJKLMNOPQRSTUVWXYZ"
@@ -99,8 +101,8 @@ static void test_create_ust_channel(void)
 
 	memset(&attr, 0, sizeof(attr));
 
-	strncpy(attr.name, "channel0", 8);
-
+	ok(lttng_strncpy(attr.name, "channel0", sizeof(attr.name)) == 0,
+		"Validate channel name length");
 	uchan = trace_ust_create_channel(&attr, LTTNG_DOMAIN_UST);
 	ok(uchan != NULL, "Create UST channel");
 
@@ -121,7 +123,9 @@ static void test_create_ust_event(void)
 	struct lttng_event ev;
 
 	memset(&ev, 0, sizeof(ev));
-	strncpy(ev.name, get_random_string(), LTTNG_SYMBOL_NAME_LEN);
+	ok(lttng_strncpy(ev.name, get_random_string(),
+			LTTNG_SYMBOL_NAME_LEN) == 0,
+		"Validate string length");
 	ev.type = LTTNG_EVENT_TRACEPOINT;
 	ev.loglevel_type = LTTNG_EVENT_LOGLEVEL_ALL;
 
@@ -143,39 +147,68 @@ static void test_create_ust_event_exclusion(void)
 	struct ltt_ust_event *event;
 	struct lttng_event ev;
 	char *name;
+	char *random_name;
 	struct lttng_event_exclusion *exclusion;
+	const int exclusion_count = 2;
 
 	memset(&ev, 0, sizeof(ev));
 
 	/* make a wildcarded event name */
 	name = get_random_string();
 	name[strlen(name) - 1] = '*';
-	strncpy(ev.name, name, LTTNG_SYMBOL_NAME_LEN);
+	ok(lttng_strncpy(ev.name, name, LTTNG_SYMBOL_NAME_LEN) == 0,
+		"Validate string length");
 
 	ev.type = LTTNG_EVENT_TRACEPOINT;
 	ev.loglevel_type = LTTNG_EVENT_LOGLEVEL_ALL;
 
 	/* set up an exclusion set */
-	exclusion = zmalloc(sizeof(*exclusion) + LTTNG_SYMBOL_NAME_LEN);
+	exclusion = zmalloc(sizeof(*exclusion) +
+		LTTNG_SYMBOL_NAME_LEN * exclusion_count);
+	ok(exclusion != NULL, "Create UST exclusion");
 	if (!exclusion) {
 		PERROR("zmalloc");
+		abort();
 	}
 
-	ok(exclusion != NULL, "Create UST exclusion");
-
-	exclusion->count = 1;
-	strncpy((char *)(exclusion->names), get_random_string(), LTTNG_SYMBOL_NAME_LEN);
+	exclusion->count = exclusion_count;
+	random_name = get_random_string();
+	strncpy(LTTNG_EVENT_EXCLUSION_NAME_AT(exclusion, 0), random_name,
+		LTTNG_SYMBOL_NAME_LEN);
+	strncpy(LTTNG_EVENT_EXCLUSION_NAME_AT(exclusion, 1), random_name,
+		LTTNG_SYMBOL_NAME_LEN);
 
 	event = trace_ust_create_event(&ev, NULL, NULL, exclusion, false);
+	exclusion = NULL;
 
-	ok(event != NULL, "Create UST event with exclusion");
+	ok(!event, "Create UST event with identical exclusion names fails");
+
+	exclusion = zmalloc(sizeof(*exclusion) +
+		LTTNG_SYMBOL_NAME_LEN * exclusion_count);
+	ok(exclusion != NULL, "Create UST exclusion");
+	if (!exclusion) {
+		PERROR("zmalloc");
+		abort();
+	}
+
+	exclusion->count = exclusion_count;
+	strncpy(LTTNG_EVENT_EXCLUSION_NAME_AT(exclusion, 0),
+		get_random_string(), LTTNG_SYMBOL_NAME_LEN);
+	strncpy(LTTNG_EVENT_EXCLUSION_NAME_AT(exclusion, 1),
+		get_random_string(), LTTNG_SYMBOL_NAME_LEN);
+
+	event = trace_ust_create_event(&ev, NULL, NULL, exclusion, false);
+	assert(event != NULL);
+
+	ok(event != NULL, "Create UST event with different exclusion names");
 
 	ok(event->enabled == 0 &&
 	   event->attr.instrumentation == LTTNG_UST_TRACEPOINT &&
 	   strcmp(event->attr.name, ev.name) == 0 &&
 	   event->exclusion != NULL &&
-	   event->exclusion->count == 1 &&
-	   strcmp((char *)(event->exclusion->names), (char *)(exclusion->names)) == 0 &&
+	   event->exclusion->count == exclusion_count &&
+	   !memcmp(event->exclusion->names, exclusion->names,
+	   	LTTNG_SYMBOL_NAME_LEN * exclusion_count) &&
 	   event->attr.name[LTTNG_UST_SYM_NAME_LEN - 1] == '\0',
 	   "Validate UST event and exclusion");
 
